'' ********************************
'' *  K107 Serial LCD Driver      *
'' *  Wulfden at Hawk's Mountain  *
'' ********************************
''
'' Driver for Wulfdem k107 Serial Controler with
'' Anderson 117 or 118 LCD Controller chips
''
'' Author.... Brian Riley
'' Updated... 21 July 2006


VAR

  long  lcdLines, lcdChars, started 


OBJ

  serial : "simple_serial"                              ' bit-bang serial driver
  numbers: "simple_numbers"
  
PUB start(pin, baud)

'' Qualifies pin, baud, 
'' -- makes tx pin an output and sets up other values if valid

  started~                                              ' clear started flag
  lcdChars~
  lcdLines~
  
  if lookdown(pin : 0..27)            ' qualify tx pin
                                      ' qualify baud rate setting
    if lookdown(baud : 2400, -2400, 9600, -9600, 19200, -19200)
      if serial.init(-1,pin, -baud)   ' sense of INV/TRU is reversed in the supporting
                                      ' serial object, so negate it here to keep K107
                                      ' people in reasonable sanity
        started~~                                    ' set flag          
            
  return started


PUB stop

'' Stop serial object, disable LCD object

  if started
    serial.finalize
    started~                                            ' set to false


PUB putc(txByte) 

'' Transmit a byte

  serial.tx(txByte)
  waitcnt(clkfreq / 1000 + cnt)                        ' slightly under 1 ms delay for pacing
   

PUB str(strAddr)

'' Transmit z-string at strAddr

  if started
    repeat strsize(strAddr)                                ' for each character in string
      serial.tx(byte[strAddr++])                           '   write the character
      

PUB cls

'' Clears LCD and moves cursor to home (0, 0) position

  if started
    str(string("?f"))
    waitcnt(clkfreq / 200 + cnt)                        ' 5 ms delay 


PUB home

'' Moves cursor to 0, 0

  if started
    str(string("?a"))
  

PUB setGeo(lines, chars)

'' Tells controller the display geometry

  if started
    if lookdown(lines: 2, 4)
      if lines == 2                                   ' check lcd size
         if lookdown(chars : 16, 20, 24, 40)           ' qualify column input
            lcdLines := 2
            lcdChars := chars
      else
        if lookdown(chars : 16, 20)
          lcdLines := 4
          lcdChars := chars
      if lcdLines > 0 and lcdChars > 0
        str(string("?G"))
        str(numbers.dec(lines))
        str(numbers.dec(chars))
 
PUB gotoxy(line, col)

'' Moves cursor to col/line

  if started
    if lcdLines > line                                  ' check lcd size
      if lcdChars > col
        str(string("?y"))
        str(numbers.decx(line,1))
        str(string("?x"))
        str(numbers.decx(col,2))

PUB clrln

'' Clears current line sets cursor in col 0

  if started
    str(string("?l"))
 

PUB bigNum3

'' Sets display to Big Number 3 column mode
'' only works for 4 lines displays

  if started
    str(string("?>3"))
 
PUB bigNum4

'' Sets display to Big Number 4 column mode
'' only works for 4 lines displays

  if started
    str(string("?>4"))
 

PUB beep

'' send 50ms 500 Hz tone on output pin

  if started
    str(string("?g"))
 
PUB back2Norm

'' brings display out of Big Num mode

  if started
    str(string("?<"))
 
PUB cursor(type)

'' Selects cursor type
''   0 : cursor off, blink off  
''   1 : cursor off, blink on   
''   2 : cursor on, blink off  
''   3 : cursor on, blink on

  if started
    if lookdown (type : 0..3)
      str(string("?c"))
      str(numbers.decx(type,1))
      waitcnt(clkfreq / 10 + cnt)                        ' 100 ms delay 
     

PUB setHi(pin)

'' Set Output pin HIGH

  if started
    if lookdown (pin : 4..7)
      str(string("?H"))
      str(numbers.decx(pin,1))

PUB setLo(pin)

'' Set Output pin LOW

  if started
    if lookdown (pin : 4..7)
      str(string("?L"))
      str(numbers.decx(pin,1))
     
     
PUB NorBackLight(level)

'' Noritake VFD Display
'' 0 - brightest
'' 1 - 75%
'' 2 - 50%
'' 3 - 25%
'' use "3-level" to get normal sense 0-3 low to high

  if started
    if lookdown (level : 0..3)
      str(string("?20?"))
      str(numbers.decx(3-level,1))

return level
 
PUB backLight(brightness)

'' 0-255 brightness command
'' -- works only with backlight-enabled displays

  if started
    if lookdown(brightness : 0..255)
      str(string("?B"))
      str(numbers.hex(brightness,2))
      waitcnt(clkfreq / 10 + cnt)                        ' 100 ms delay 


  return brightness  
  

  
